@@ -1,5 +1,7 @@
 package Hash::AutoHash::Args;
-our $VERSION='1.00';
+our $VERSION='1.14';
+$VERSION=eval $VERSION;		# I think this is the accepted idiom..
+
 #################################################################################
 #
 # Author:  Nat Goodman
@@ -43,6 +45,7 @@ our @SUBCLASS_EXPORT_OK=$helper_class->SUBCLASS_EXPORT_OK;
 # functions herein (except _new) are exportable by Hash::AutoHash::Args
 #################################################################################
 package Hash::AutoHash::Args::helper;
+our $VERSION='1.14';
 use strict;
 use Carp;
 use Scalar::Util qw(reftype);
@@ -108,7 +111,9 @@ sub fix_args {
     my($keyword,$value)=(fix_keyword(shift @args),shift @args);
     $args->{$keyword}=$value,next unless exists $args->{$keyword};
     my $old=$args->{$keyword};
-    $args->{$keyword}=[$old,$value],next unless ref $old; # grow scalar slot into ARRAY
+    # NG 09-12-31: breaks if $old is object. 
+    # $args->{$keyword}=[$old,$value],next unless ref $old; # grow scalar slot into ARRAY
+    $args->{$keyword}=[$old,$value],next unless 'ARRAY' eq ref $old; # grow scalar slot into ARRAY
     push(@$old,$value);					  # else add new value to ARRAY slot
   }
   $args;
@@ -129,7 +134,7 @@ sub is_positional {@_%2 || $_[0]!~/^-/;}
 # Tied hash which provides the core capabilities of Hash::AutoHash::Args
 #################################################################################
 package Hash::AutoHash::Args::tie;
-my $VERSION='1.0';
+our $VERSION='1.14';
 use strict;
 use Carp;
 use Tie::Hash;
@@ -173,7 +178,11 @@ __END__
 
 =head1 NAME
 
-Hash::AutoHash::Args - Argument list processing
+Hash::AutoHash::Args - Object-oriented processing of keyword-based argument lists
+
+=head1 VERSION
+
+Version 1.14
 
 =head1 SYNOPSIS
 
@@ -295,7 +304,7 @@ By aliasing $args to %args, you avoid the need to dereference the
 variable when using hash notation.  Admittedly, this is a minor
 convenience, but then again, this entire class is about convenience.
 
-=head2 Constructors
+=head2 new
 
  Title   : new
  Usage   : $args=new Hash::AutoHash::Args
@@ -416,6 +425,8 @@ unfortunately, Perl does not support this behavior with HASH notation.
 
 =back
 
+=head2 Wholesale manipulation of arguments
+
 The class also provides several functions for wholesale manipulation of
 arguments. To use these functions, you must import them into the
 caller's namespace using the common Perl idiom of listing the desired
@@ -424,12 +435,16 @@ functions in a 'use' statement.  For example,
  use Hash::AutoHash::Args
     qw(get_args getall_args set_args autoargs_get autoargs_set);
  
+=head3 get_args
+
  Title   : get_args
  Usage   : ($name,$hobbies)=get_args($args,qw(-name hobbies))
  Function: Get values for multiple keywords
  Args    : Hash::AutoHash::Args object and array or ARRAY of keywords
  Returns : array or ARRAY of argument values
 
+=head3 autoargs_get
+
  Title   : autoargs_get
  Usage   : ($name,$hobbies)=autoargs_get($args,qw(name -hobbies))
  Function: Get values for multiple keywords. Synonym for 'get_args' provided 
@@ -437,12 +452,16 @@ functions in a 'use' statement.  For example,
  Args    : Hash::AutoHash::Args object and array or ARRAY of keywords
  Returns : array or ARRAY of argument values
 
+=head3 getall_args
+
  Title   : getall_args
  Usage   : %args=getall_args($args);
  Function: Get all keyword, value pairs
  Args    : Hash::AutoHash::Args object
  Returns : hash or HASH of key=>value pairs.
 
+=head3 set_args
+
  Title   : set_args
  Usage   : set_args($args,
                     name=>'Joe the Plumber',-first_name=>'Joe',-last_name=>'Plumber')
@@ -456,6 +475,8 @@ functions in a 'use' statement.  For example,
            and values
  Returns : nothing
 
+=head3 autoargs_set
+
  Title   : autoargs_set
  Usage   : autoargs_set($args,
                         name=>'Joe the Plumber',-first_name=>'Joe',-last_name=>'Plumber')
@@ -471,7 +492,7 @@ functions in a 'use' statement.  For example,
            and values
  Returns : Hash::AutoHash::Args object
 
-=head2 Aliasing object to hash
+=head2 Aliasing object to hash: autoargs_alias
 
 You can alias a Hash::AutoHash::Args object to a regular hash to
 avoid the need to dereference the variable when using hash
@@ -495,6 +516,8 @@ using the common Perl idiom of listing the desired functions in a
 'use' statement.
 
  use Hash::AutoHash::Args qw(fix_args fix_keyword fix_keywords);
+
+=head3 fix_args
  
  Title   : fix_args
  Usage   : $hash=fix_args(-name=>'Joe',HOBBIES=>'hiking',hobbies=>'cooking')
@@ -504,6 +527,8 @@ using the common Perl idiom of listing the desired functions in a
            'autoargs_set'.  
  Returns : HASH of normalized keyword=>value pairs
 
+=head3 fix_keyword
+ 
  Title   : fix_keyword
  Usage   : $keyword=fix_keyword('-NaMe')
            -- OR --
@@ -512,6 +537,8 @@ using the common Perl idiom of listing the desired functions in a
  Args    : array of one or more strings
  Returns : array of normalized strings
 
+=head3 fix_keywords
+ 
  Title   : fix_keywords
  Usage   : $keyword=fix_keywords('-NaMe')
            -- OR --
@@ -530,6 +557,8 @@ To use these functions, they must be imported into the caller's namespace.
 
  use Hash::AutoHash::Args qw(is_keyword is_positional);
 
+=head3 is_keyword
+
  Title   : is_keyword
  Usage   : if (is_keyword(@args)) {
              $args=new Hash::AutoHash::Args (@args);
@@ -542,6 +571,8 @@ To use these functions, they must be imported into the caller's namespace.
  Returns : boolean
  Args    : argument list as given
 
+=head3 is_positional
+
  Title   : is_positional
  Usage  : if (is_positional(@args)) {
              ($arg1,$arg2,$arg3)=@args; 
@@ -591,12 +622,16 @@ To use these functions, you must imported then into the caller's namespace, eg,
                                autoargs_keys autoargs_values 
                                autoargs_count autoargs_empty autoargs_notempty);
 
+=head3 autoargs_clear
+
  Title   : autoargs_clear
  Usage   : autoargs_clear($args)
  Function: Delete entire contents of $args
  Args    : Hash::AutoHash::Args object
  Returns : nothing
 
+=head3 autoargs_delete
+
  Title   : autoargs_delete
  Usage   : autoargs_delete($args,@keywords)
  Function: Delete keywords and their values from $args. The keywords are
@@ -604,6 +639,8 @@ To use these functions, you must imported then into the caller's namespace, eg,
  Args    : Hash::AutoHash::Args object, list of keywords
  Returns : nothing
 
+=head3 autoargs_exists
+
  Title   : autoargs_exists
  Usage   : if (autoargs_exists($args,$keyword)) { ... }
  Function: Test whether keyword is present in $args.  The keyword is
@@ -611,6 +648,8 @@ To use these functions, you must imported then into the caller's namespace, eg,
  Args    : Hash::AutoHash::Args object, keyword
  Returns : boolean
 
+=head3 autoargs_each
+
  Title   : autoargs_each
  Usage   : while (my($keyword,$value)=autoargs_each($args)) { ... }
            -- OR --
@@ -620,30 +659,40 @@ To use these functions, you must imported then into the caller's namespace, eg,
  Returns : list context: next keyword=>value pair in $args or empty list at end
            scalar context: next keyword in $args or undef at end
 
+=head3 autoargs_keys
+
  Title   : autoargs_keys
  Usage   : @keys=autoargs_keys($args)
  Function: Get all keywords that are present in $args
  Args    : Hash::AutoHash::Args object
  Returns : list of keywords
 
+=head3 autoargs_values
+
  Title   : autoargs_values
  Usage   : @values=autoargs_values($args)
  Function: Get the values of all keywords that are present in $args
  Args    : Hash::AutoHash::Args object
  Returns : list of values
 
+=head3 autoargs_count
+
  Title   : autoargs_count
  Usage   : $count=autoargs_count($args)
  Function: Get the number keywords that are present in $args
  Args    : Hash::AutoHash::Args object
  Returns : number
 
+=head3 autoargs_empty
+
  Title   : autoargs_empty
  Usage   : if (autoargs_empty($args) { ... }
  Function: Test whether $args is empty
  Args    : Hash::AutoHash::Args object
  Returns : boolean
 
+=head3 autoargs_notempty
+
  Title   : autoargs_notempty
  Usage   : if (autoargs_notempty($args) { ... }
  Function: Test whether $args is not empty. Complement of autoargs_empty
@@ -674,36 +723,25 @@ were inherited from UNIVERSAL (the base class of everything, e.g,
 
 Hash::AutoHash::Args has a cleaner namespace: B<no keywords are masked>.
 
+CAUTION: As of version 1.13, it is not possible to use method
+notation for keys with the same names as methods inherited from
+UNIVERSAL (the base class of everything). These are 'can', 'isa',
+'DOES', and 'VERSION'.  The reason is that as of Perl 5.9.3, calling
+UNIVERSAL methods as functions is deprecated and developers are
+encouraged to use method form instead. Previous versions of AutoHash
+are incompatible with CPAN modules that adopt this style.
+
 =item * Object vs. class methods
 
-Some of the methods that remain in the Hash::AutoHash::Args
-namespace are logically object-methods (ie, logically apply to
-individual Hash::AutoHash::Args objects, eg, 'AUTOLOAD'), while
-others are logically class-methods (ie, apply to the entire class, eg,
-'can', 'import').
+Some of the methods that remain in the Hash::AutoHash::Args namespace
+are logically object-methods (ie, logically apply to individual
+Hash::AutoHash::Args objects, eg, 'AUTOLOAD'), while others are
+logically class-methods (ie, apply to the entire class, eg, 'import').
 
 For methods that are logically class methods, the code checks whether
 the method was invoked on a class or an object and "does the right
 thing".  
 
-For example, the following two statements
-
-  can Hash::AutoHash::Args('import')
-  Hash::AutoHash::Args->can('import')
-
-invoke the 'can' method on the class Hash::AutoHash::Args; the code
-notices that the target is a class and forwards the call to the 'can'
-method in UNIVERSAL which tells whether the module has an 'import'
-method (it does).  On the other hand,
-
-  $args->can('import')
-
-invokes the 'can' method on the $args object (assuming, of course,
-that's what $args contains); the code notices that the target is a
-Hash::AutoHash::Args object and sets the value of the keyword 'can'
-to 'import'.  This may sound confusing, but it seems to work naturally
-in practice.
-
 The 'new' method warrants special mention.  In normal use, 'new' is
 almost always invoked as a class method, eg, 
 
@@ -776,8 +814,8 @@ L<Class::AutoClass::Args> is replaced by this
 class. L<Hash::AutoHash::Args::V0> is a subclass which is more
 compatible with Class::AutoClass::Args.
 
-L<Hash::AutoHash::MultiValued>, L<Hash::AutoHash::AVP_Single>,
-L<Hash::AutoHash::AVP_Multi>, L<Hash::AutoHash::Record> are other
+L<Hash::AutoHash::MultiValued>, L<Hash::AutoHash::AVPairsSingle>,
+L<Hash::AutoHash::AVPairsMulti>, L<Hash::AutoHash::Record> are other
 subclasses of L<Hash::AutoHash>.
 
 L<perltie> and L<Tie::Hash> present background on tied hashes.
